#include "yuv.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>

static double get_time(void);

static double
get_time(void)
{
   struct timeval      timev;
   
   gettimeofday(&timev, NULL);
   return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
}

void
load_yuv(char *file, unsigned char *data)
{
   FILE *f;
   
   f = fopen(file, "r");
   if (f)
     {
	fread(data, 720 * 480 * 3 / 2, 1, f);
	fclose(f);
     }
   else
     {
	printf("ERROR: cannot load %s\n", file);
     }
}
void
save_ppm(char *file, unsigned char *data)
{
   FILE *f;
   
   f = fopen(file, "w");
   if (f)
     {
	int x, y;
	unsigned char *p;
	
	fprintf(f, "P6\n720 480\n255\n");
	p = data;
	for (y = 0; y < 480; y++)
	  {
	     for (x = 0; x < 720; x++)
	       {
		  unsigned char i;
		  
		  i = R_VAL(p);
		  fwrite(&i, 1, 1, f);
		  i = G_VAL(p);
		  fwrite(&i, 1, 1, f);
		  i = B_VAL(p);
		  fwrite(&i, 1, 1, f);
		  p += 4;
	       }
	  }
	fclose(f);
     }
   else
     {
	printf("ERROR: cannot write %s\n", file);
     }
}

int 
main(int argc, char **argv)
{
   int i;
   unsigned char *rgb, *yuv;
   double t1, t2;
   
   int test_mode = 1;
   int loops = 1024;
   
   
   for (i = 1; i < argc; i++)
     {
	if (!strcmp(argv[i], "-l") && (argc > (i + 1)))
	  {
	     i++;
	     loops = atoi(argv[i]);
	  }
	if (!strcmp(argv[i], "-t") && (argc > (i + 1)))
	  {
	     i++;
	     if (!strcmp(argv[i], "memcpy"))     test_mode = 0;
	     if (!strcmp(argv[i], "raster"))     test_mode = 1;
	     if (!strcmp(argv[i], "diz"))        test_mode = 2;
	     if (!strcmp(argv[i], "mmx"))        test_mode = 3;
	  }
	if (!strcmp(argv[i], "-h"))
	  {
	     printf("usage:\n"
		    "\tyuvrgb [ options ]\n"
		    "where options is zero or more of:n"
		    "\t-l number_of_loops\n"
		    "\t-t test_mode\n"
		    "\t   test mode can be one of:\n"
		    "\t     memcpy, raster, diz, mmx\n");
	     exit(0);
	  }
     }
   
   yuv_init(8, 8, 8);
   
   yuv = malloc(720 * 480 * 4);
   rgb = malloc(720 * 480 * 4);

   /* load test data */
   load_yuv("test.yuv", yuv);
     
   printf("Test... : ");
   fflush(stdout);
   
   t1 = get_time();
   for (i = 0; i < loops; i++)
     {
	if (test_mode == 0)
	  memcpy(rgb, yuv, 720 * 480 * ((4 + (3 / 2)) / 2));
	else if (test_mode == 1)
	  yv12torgb_raster(yuv, rgb, 720, 480);
	else if (test_mode == 2)
	  yv12torgb_diz(yuv, rgb, 720, 480);
	else if (test_mode == 3)
	  yv12torgb_mmx(yuv, rgb, 720, 480);
     }
   t2 = get_time();
   
   save_ppm("out.ppm", rgb);
   
   printf("%3.3ffps (%3.3f sec for 1024 frames of 720x480)\n", 
	  (double)((double)i / (t2 - t1)),
	  t2 - t1);
}

